`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: Renesas
// Engineer: Andrii Oksenchuk
// 
// Create Date: 05/17/2022 02:30:08 PM
// Design Name: 
// Module Name: BCD
// Project Name: Hronograf
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////


module BCD(
    input clk,
    input [13:0] binary_in,
    input enable_conver,
    output reg [3:0] ones = 0,
    output reg [3:0] tens = 0,
    output reg [3:0] hundrets = 0,
    output reg [3:0] thousands = 0,
    output reg tens_of_thousands = 0
    );
    
reg[3:0] i = 0;
reg[30:0] shift_register = 0;

reg[3:0] temp_thousands = 0;
reg[3:0] temp_hundrets = 0;
reg[3:0] temp_tens = 0;
reg[3:0] temp_ones = 0;
reg temp_tens_of_thousands = 0;


always@ (posedge clk)
begin
    
    if ( i == 0 & enable_conver) begin
        
        shift_register[13:0] = binary_in;
        shift_register[30:14] = 0;
        
        temp_thousands = 0;
        temp_hundrets = 0; 
        temp_tens = 0;     
        temp_ones = 0; 
        
        i = i + 1;    
    end
    
    if ( i < 15 & i > 0) begin
        if (temp_thousands >= 5) temp_thousands = temp_thousands + 3;
        if (temp_hundrets >= 5) temp_hundrets = temp_hundrets + 3;
        if (temp_tens >= 5) temp_tens = temp_tens + 3;
        if (temp_ones >= 5) temp_ones = temp_ones + 3;
        
        shift_register [29:14] = {temp_thousands, temp_hundrets, temp_tens, temp_ones};
        shift_register = shift_register << 1;
        
        temp_tens_of_thousands = shift_register [30];
        temp_thousands = shift_register [29:26];
        temp_hundrets = shift_register [25:22];
        temp_tens = shift_register [21:18];
        temp_ones = shift_register [17:14];
        
        i = i + 1;
    end
    
    if (i == 15) begin
        i = 0;
        
        tens_of_thousands = temp_tens_of_thousands;
        thousands = temp_thousands;
        hundrets = temp_hundrets;
        tens = temp_tens;
        ones = temp_ones;
        
        shift_register[30:14] = 0;
    end

end



endmodule

`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date: 05/17/2022 03:52:41 PM
// Design Name: 
// Module Name: TOP
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////


(* top *) module TOP #(
/* --------------------------- Set hardware parameter --------------------------- */
	parameter OSC_frequency = 50000000,
	parameter distance_between_sensors = 0.3
	)(
   	(* iopad_external_pin, clkbuf_inhibit *) 	input clk,
   	(* iopad_external_pin *) 					input nreset,
   	(* iopad_external_pin *)					input start,
   	(* iopad_external_pin *)					output start_oe,
   	(* iopad_external_pin *)					input stop,
   	(* iopad_external_pin *)					output stop_oe,
   	(* iopad_external_pin *)					output [7:0] anods,
   	(* iopad_external_pin *)					output [3:0] catods,
   
  	// OSC config outputs
	(* iopad_external_pin *) output osc_en,
  	(* iopad_external_pin *) output osc_mode
    );

/* --------------------------- calculating patameter --------------------------- */
/*
* The value distance_for_calculating_speed depends on the distance between the sensors and the frequency of the ?lk. 
* distance = (distance between sensors in meters) * (frequency clk in Hz) * 10(speed measure with step 0.1 m/c)
* 150000000 for distance between sensors = 0.3 m, frequency = 50Mhz
*/
parameter distance_for_calculating_speed = distance_between_sensors * OSC_frequency * 10;
/*
* if the time between the operation of the sensors is equivalent to a speed of more than 1621 m/s, 
* the display will show a speed of 0 m/s
*/
parameter clock_for_max_speed = distance_for_calculating_speed / 16210; 


assign osc_en = 1'b1;
assign osc_mode = 1'b1;

assign start_oe = 1'b1;
assign stop_oe = 1'b1;


reg [13:0] data_binary = 0;
wire [3:0] ones;
wire [3:0] tens;
wire [3:0] hundrets;
wire [3:0] thousands;
wire tens_of_thousands;
reg lcd_reset = 0;
reg enable_BCD_conver = 0;

BCD u1(
.clk(clk), 
.binary_in(data_binary), 
.enable_conver(enable_BCD_conver),
.ones(ones),
.tens(tens), 
.hundrets(hundrets), 
.thousands(thousands), 
.tens_of_thousands(tens_of_thousands)
);

four_digits_7_segment u2 (
.clk(clk), 
.reset(lcd_reset), 
.tens_of_thousands(tens_of_thousands), 
.thousands(thousands), 
.hundrets(hundrets), 
.tens(tens), 
.ones(ones), 
.anods(anods), 
.catods(catods)
);



parameter idle = 3'b000, wait_stop = 3'b001, check_data = 3'b010, calculate_speed = 3'b011, BCD_convert = 3'b100, wait_afret_measure = 3'b101, init = 3'b111;
reg [2:0] state = init;

reg [27:0] counter = 0;
reg [31:0] wait_counter = 0;

reg [27:0] distance = distance_for_calculating_speed;


always@ (posedge clk)
begin
	if(!nreset)
		lcd_reset = 1;
    else begin
    	case(state)
    		idle : begin
    			if(lcd_reset == 0) begin
    			/* if wait_counter == 3_221_225_472, wait for ~64 c before resetting the LCD, after measure(if ?lk == 50MHz) */
            		if (wait_counter[31:30] == 2'b11) begin  
                		lcd_reset = 1;
            		end else begin
                		lcd_reset = 0;
                		wait_counter = wait_counter + 1;
            		end
            	end
    		end
    		
    		wait_stop : begin
    			wait_counter = 0;
    			lcd_reset = 1;
    		end	
    	
    		BCD_convert :
    			lcd_reset = 0;
    		wait_afret_measure :
    			lcd_reset = 0;	
    		default : 
    			lcd_reset = 1;
    	endcase
    end
end

always@ (posedge clk)
begin
    if(!nreset) begin
        counter = 0;
        state = init;
    end else begin
        case(state)
        init : begin
            counter = counter + 1; 
            /* ~1c start delay (if clk == 50Mhz), need for init optical sensor
            * use counter[25:24] == 2'b11 for synthesis
            * use counter[5:4] == 2'b11 for simulation
            */
            if (counter[5:4] == 2'b11)
                state = idle;
        end
        
        idle : begin 
        
            distance = distance_for_calculating_speed;
            data_binary = 0;
            counter = 0;
            
            if (start == 1)
                state = wait_stop;
        end
        
        wait_stop : begin
        
            counter = counter + 1; 
            
            if (stop == 1)
                state = check_data;
            else
            	/* if the stop signal is not received after 16_777_216 clocks(if clk == 50 MHz equivalent speed less than 1m/c), going to idle */
            	if (counter[24] == 1)
                	state = idle;
              	else
              		state = wait_stop;
        end
        
        check_data : begin

            if (counter < clock_for_max_speed)  
                counter = 28'hFFFFFFF;
            else 
                state = calculate_speed;
        end
        
        calculate_speed : begin
            if (distance >= counter) begin
                distance = distance - counter;
                data_binary = data_binary + 1;
            end else begin
                counter = 0;
                state = BCD_convert;
            end
        end
        
        BCD_convert : begin
  
            counter = counter + 1;
            
            if (counter[1:0] == 2'b01)
                enable_BCD_conver = 1;
                
            if (counter[1:0] == 2'b11) begin
                enable_BCD_conver = 0;
                state = wait_afret_measure;
            end else
            	state = BCD_convert;
                
        end
        wait_afret_measure : begin
        
        	counter = counter + 1;
        	/* wait for 2c after measure */
        	if (counter[26:25] == 2'b11) begin
        		state = idle;
            end else begin
            	state = wait_afret_measure;
            end
        
        end
        endcase
    end
end

endmodule

`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date: 05/20/2022 10:36:15 AM
// Design Name: 
// Module Name: 4_digits_7_segment
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////


module four_digits_7_segment(
    input clk,
    input reset,
    input tens_of_thousands,
    input [3:0] thousands,
    input [3:0] hundrets,
    input [3:0] tens,
    input [3:0] ones,
    output reg [7:0] anods,
    output reg [3:0] catods
    );
    
    
    
reg [16:0] clk_counter = 0;
reg [1:0] catod_number = 0;
reg [3:0] number = 0;
reg dot = 0;


/* clock divider */

always@ (posedge clk)
begin
    if (clk_counter[16:15] == 2'b11) begin
    
        clk_counter = 0;
        
        if (catod_number == 3) begin
            catod_number = 0;
        end else begin
            catod_number = catod_number + 1;
        end
        
    end else begin
        clk_counter = clk_counter + 1;
    end
end
    

always@  (catod_number) begin // , catod_number
    case(catod_number)
        0 : catods = 4'b1110;
        1 : catods = 4'b1101;
        2 : catods = 4'b1011;
        3 : catods = 4'b0111;
    endcase
end
    
always@ (catod_number) begin // catod_number
    if (reset == 1) begin
        number = 10;
        dot = 0;
    end else begin
        if (tens_of_thousands == 1) begin
            dot = 0;
            case(catod_number)
                2'b00 : number = tens;
                2'b01 : number = hundrets;
                2'b10 : number = thousands;
                2'b11 : number = 1;
            endcase
        end else begin
            dot = 1;
            case(catod_number)
                0 : number = ones;
                1 : number = tens;
                2 : number = hundrets;
                3 : number = thousands;
            endcase
        end
    end
end
    
always@ (number) begin  // , catod_number, number
    
    if (catod_number == 1)
        anods[0] = dot;
    else
        anods[0] = 0;
    
    case(number)
        0 : anods[7:1] = 7'b1110111;
        1 : anods[7:1] = 7'b0010100;
        2 : anods[7:1] = 7'b1011011;
        3 : anods[7:1] = 7'b1011110;
        4 : anods[7:1] = 7'b0111100;
        5 : anods[7:1] = 7'b1101110;
        6 : anods[7:1] = 7'b1101111;
        7 : anods[7:1] = 7'b1010100;
        8 : anods[7:1] = 7'b1111111;
        9 : anods[7:1] = 7'b1111110;
        default : anods[7:1] = 7'b0001000;
    endcase
    
end    
    
    
endmodule



